#ifndef MINIAUDIOENGINE_H
#define MINIAUDIOENGINE_H

#define MA_ENABLE_ONLY_SPECIFIC_BACKENDS 1
#define MA_ENABLE_JACK 1
#define MA_NO_GENERATION 1
#define MA_DEBUG_OUTPUT 1
#define MA_DISABLE_PULSEAUDIO
#define MINIAUDIO_IMPLEMENTATION
#include "miniaudio.h"
#include "defines.h" // MAX_LAYERS
#include <bits/stdc++.h>
using namespace std;

/* Data Format */
#define FORMAT              ma_format_f32   /* Must always be f32. */
#define CHANNELS            2
#define SAMPLE_RATE         48000

typedef struct
{
    ma_hpf_node hpf;
    ma_hpf_config hpfConfig;
    ma_lpf_node lpf;
    ma_lpf_config lpfConfig;
    ma_peak_node mLow;
    ma_peak_config mLowConfig;
    ma_peak_node mHigh;
    ma_notch_node notch;
    ma_peak_config mHighConfig;
} filterBank;


class MiniAudioEngine
{
    friend class libreMediaServerAudio;

public:
    MiniAudioEngine();
    void stopEngine();
    bool startEngine(uint layersQty);
    bool startDevice(uint *id, uint nb);
    static void audioDataCallback(ma_device* pDevice, void* pOutput, const void* pInput, ma_uint32 frameCount);

protected:
    ma_result loadMedia(int layer, char *media, uint audioDevice);
    void volChanged(int layer, float vol);
    void panChanged(int layer, float pan);
    void pitchChanged(int layer, float pitch);
    ma_result playbackChanged(int layer, Status status);
    ma_result setCursor(int layer, int cursor);
    ma_result printFormatInfo(int layer);
    float getDuration(int layer);
    float getCursor(int layer);
    Status getStatus(int layer);
    inline float getVol(int layer) {
        return ma_sound_get_volume(&m_currentSound[layer]); }
    inline bool getAtEnd(int layer) { return m_currentSound[layer].atEnd; }
    ma_result filterParamChanged(int layer, int audioDevice, int channel, int value);

private:
    ma_resource_manager_config m_resourceManagerConfig;
    ma_resource_manager m_resourceManager;
    ma_context m_context;
    ma_device_info* m_pPlaybackDeviceInfos;
    ma_uint32 m_playbackDeviceCount;
    ma_uint32 m_devicesSelected;
    ma_device m_device[MAX_AUDIODEVICES];
    ma_sound m_currentSound[MAX_LAYERS];
    ma_bool8 m_mediaLoaded[MAX_LAYERS];
    layerData m_currentLayerValues[MAX_LAYERS];
    filterBank m_filterBank[MAX_AUDIODEVICES][MAX_LAYERS];
    ma_engine m_engine[MAX_AUDIODEVICES];
    uint m_layersQty;

    ma_result getAllAudioDevices();
    ma_result startContext();
    void refreshValues(int layer);
    ma_result seekToCursor(int layer, int cursor);
    ma_result setNodeGraph(int id);
    ma_result createFilterBank(int id, uint layer);
};

#endif // MINIAUDIOENGINE_H
